package edu.mit.harold.icollections.tests;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import edu.mit.harold.icollections.IVector;
import edu.mit.harold.icollections.TreeIVector;


import junit.framework.TestCase;

public class TreeIVectorTest extends TestCase {

	/**
	 * Compares the behavior of java.util.LinkedList to the behavior of TreeIVector.
	 */
	public void testRandomlyAgainstJavaList() {
		IVector<Integer> ivec = TreeIVector.empty();
		List<Integer> list = new LinkedList<Integer>();
		Random r = new Random();
		for(int i=0;i<1000;i++) {
			if(ivec.size()==0 || r.nextBoolean()) { // add
				if(r.nextBoolean()) { // append
					Integer v = r.nextInt();
		
					assertEquals(list.contains(v), ivec.contains(v));
					
					list.add(v);
					ivec = ivec.plus(v);
				}
				else { // insert
					int k = r.nextInt(ivec.size()+1);
					Integer v = r.nextInt();
	
					assertEquals(list.contains(v), ivec.contains(v));
					if(k<ivec.size())
						assertEquals(list.get(k), ivec.get(k));
	
					list.add(k, v);
					ivec = ivec.plus(k, v);
				}
			}
			else if(r.nextBoolean()) { // replace
				int k = r.nextInt(ivec.size());
				Integer v = r.nextInt();
				list.set(k, v);
				ivec = ivec.with(k, v);
			}
			else { // remove a random element
				int j = r.nextInt(ivec.size()), k=0;
				for(Integer e : ivec) {

					assertTrue(list.contains(e));
					assertTrue(ivec.contains(e));
					assertEquals(e, ivec.get(k));
					assertEquals(list.get(k), ivec.get(k));
					UtilityTest.assertEqualsAndHash( ivec, ivec.minus(k).plus(k, ivec.get(k)) );
					UtilityTest.assertEqualsAndHash( ivec, ivec.plus(k, 10).minus(k) );

					if(k==j) {
						list.remove(k);
						ivec = ivec.minus(k);
						k--; // indices are now smaller
						j = -1; // don't remove again
					}
					k++;
				}
			}
			
			// also try to remove a _totally_ random value:
			Integer v = r.nextInt();
			assertEquals(list.contains(v), ivec.contains(v));
			list.remove(v);
			ivec = ivec.minus(v);
			
			// and try out a non-Integer:
			String s = Integer.toString(v);
			assertFalse( ivec.contains(v) );
			ivec = ivec.minus(s);

			assertEquals(list.size(), ivec.size());
			UtilityTest.assertEqualsAndHash(list, ivec);
			
			UtilityTest.assertEqualsAndHash(ivec, TreeIVector.from(ivec));
			UtilityTest.assertEqualsAndHash(TreeIVector.empty(), ivec.minusAll(ivec));
			UtilityTest.assertEqualsAndHash(ivec, TreeIVector.empty().plusAll(ivec));
			UtilityTest.assertEqualsAndHash(ivec, TreeIVector.singleton(10).plusAll(1,ivec).minus(0));
			
			int end=r.nextInt(ivec.size()+1), start=r.nextInt(end+1);
			UtilityTest.assertEqualsAndHash(ivec.subList(start, end), list.subList(start, end));
		}
	}

	public void testSubListIntInt() {
		// TODO
	}

	public void testMinusObject() {
		// TODO
	}

}
